"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertArrayTypeNode = exports.isArrayTypeNode = exports.arrayTypeNodeFromIdl = exports.arrayTypeNode = void 0;
const SizeStrategy_1 = require("../shared/SizeStrategy");
const NumberTypeNode_1 = require("./NumberTypeNode");
const TypeNode_1 = require("./TypeNode");
function arrayTypeNode(child, options = {}) {
    var _a;
    return {
        kind: 'arrayTypeNode',
        child,
        size: (_a = options.size) !== null && _a !== void 0 ? _a : (0, SizeStrategy_1.prefixedSize)(),
    };
}
exports.arrayTypeNode = arrayTypeNode;
function arrayTypeNodeFromIdl(idl) {
    if ('array' in idl) {
        const child = (0, TypeNode_1.createTypeNodeFromIdl)(idl.array[0]);
        return arrayTypeNode(child, { size: (0, SizeStrategy_1.fixedSize)(idl.array[1]) });
    }
    const child = (0, TypeNode_1.createTypeNodeFromIdl)(idl.vec);
    if (!idl.size)
        return arrayTypeNode(child);
    if (idl.size === 'remainder') {
        return arrayTypeNode(child, { size: (0, SizeStrategy_1.remainderSize)() });
    }
    return arrayTypeNode(child, {
        size: (0, SizeStrategy_1.prefixedSize)((0, NumberTypeNode_1.numberTypeNode)(idl.size)),
    });
}
exports.arrayTypeNodeFromIdl = arrayTypeNodeFromIdl;
function isArrayTypeNode(node) {
    return !!node && node.kind === 'arrayTypeNode';
}
exports.isArrayTypeNode = isArrayTypeNode;
function assertArrayTypeNode(node) {
    var _a;
    if (!isArrayTypeNode(node)) {
        throw new Error(`Expected arrayTypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertArrayTypeNode = assertArrayTypeNode;
//# sourceMappingURL=ArrayTypeNode.js.map