"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertStructFieldTypeNode = exports.isStructFieldTypeNode = exports.structFieldTypeNodeFromIdl = exports.structFieldTypeNode = void 0;
const shared_1 = require("../shared");
const TypeNode_1 = require("./TypeNode");
const ValueNode_1 = require("./ValueNode");
function structFieldTypeNode(input) {
    var _a, _b;
    if (!input.name) {
        throw new shared_1.InvalidKinobiTreeError('StructFieldTypeNode must have a name.');
    }
    return {
        kind: 'structFieldTypeNode',
        name: (0, shared_1.mainCase)(input.name),
        child: input.child,
        docs: (_a = input.docs) !== null && _a !== void 0 ? _a : [],
        defaultsTo: (_b = input.defaultsTo) !== null && _b !== void 0 ? _b : null,
    };
}
exports.structFieldTypeNode = structFieldTypeNode;
function structFieldTypeNodeFromIdl(idl) {
    var _a, _b;
    return structFieldTypeNode({
        name: (_a = idl.name) !== null && _a !== void 0 ? _a : '',
        child: (0, TypeNode_1.createTypeNodeFromIdl)(idl.type),
        docs: (_b = idl.docs) !== null && _b !== void 0 ? _b : [],
        defaultsTo: idl.defaultsValue !== undefined
            ? { strategy: 'optional', value: (0, ValueNode_1.vScalar)(idl.defaultsValue) }
            : null,
    });
}
exports.structFieldTypeNodeFromIdl = structFieldTypeNodeFromIdl;
function isStructFieldTypeNode(node) {
    return !!node && node.kind === 'structFieldTypeNode';
}
exports.isStructFieldTypeNode = isStructFieldTypeNode;
function assertStructFieldTypeNode(node) {
    var _a;
    if (!isStructFieldTypeNode(node)) {
        throw new Error(`Expected structFieldTypeNode, got ${(_a = node === null || node === void 0 ? void 0 : node.kind) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertStructFieldTypeNode = assertStructFieldTypeNode;
//# sourceMappingURL=StructFieldTypeNode.js.map