"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetJavaScriptRenderMapVisitor = void 0;
const prettier_1 = require("prettier");
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const logs_1 = require("../../shared/logs");
const visitors_1 = require("../../visitors");
const RenderMap_1 = require("../RenderMap");
const utils_1 = require("../utils");
const GetJavaScriptTypeManifestVisitor_1 = require("./GetJavaScriptTypeManifestVisitor");
const JavaScriptImportMap_1 = require("./JavaScriptImportMap");
const RenderJavaScriptValueNode_1 = require("./RenderJavaScriptValueNode");
const DEFAULT_PRETTIER_OPTIONS = {
    semi: true,
    singleQuote: true,
    trailingComma: 'es5',
    useTabs: false,
    tabWidth: 2,
    arrowParens: 'always',
    printWidth: 80,
    parser: 'typescript',
};
class GetJavaScriptRenderMapVisitor extends visitors_1.BaseThrowVisitor {
    constructor(options = {}) {
        var _a, _b, _c, _d, _e;
        super();
        this.program = null;
        this.options = {
            renderParentInstructions: (_a = options.renderParentInstructions) !== null && _a !== void 0 ? _a : false,
            formatCode: (_b = options.formatCode) !== null && _b !== void 0 ? _b : true,
            prettierOptions: {
                ...DEFAULT_PRETTIER_OPTIONS,
                ...options.prettierOptions,
            },
            dependencyMap: {
                generated: '..',
                hooked: '../../hooked',
                umi: '@metaplex-foundation/umi',
                umiSerializers: '@metaplex-foundation/umi/serializers',
                mplEssentials: '@metaplex-foundation/mpl-toolbox',
                mplToolbox: '@metaplex-foundation/mpl-toolbox',
                ...options.dependencyMap,
                // Custom relative dependencies to link generated files together.
                generatedAccounts: '../accounts',
                generatedErrors: '../errors',
                generatedTypes: '../types',
            },
            typeManifestVisitor: (_c = options.typeManifestVisitor) !== null && _c !== void 0 ? _c : new GetJavaScriptTypeManifestVisitor_1.GetJavaScriptTypeManifestVisitor(),
            byteSizeVisitor: (_d = options.byteSizeVisitor) !== null && _d !== void 0 ? _d : new visitors_1.GetByteSizeVisitor(),
            resolvedInstructionInputVisitor: (_e = options.resolvedInstructionInputVisitor) !== null && _e !== void 0 ? _e : new visitors_1.GetResolvedInstructionInputsVisitor(),
        };
    }
    visitRoot(root) {
        var _a, _b;
        (_b = (_a = this.byteSizeVisitor).registerDefinedTypes) === null || _b === void 0 ? void 0 : _b.call(_a, nodes.getAllDefinedTypes(root));
        const programsToExport = root.programs.filter((p) => !p.internal);
        const accountsToExport = nodes
            .getAllAccounts(root)
            .filter((a) => !a.internal);
        const instructionsToExport = nodes
            .getAllInstructionsWithSubs(root, !this.options.renderParentInstructions)
            .filter((i) => !i.internal);
        const definedTypesToExport = nodes
            .getAllDefinedTypes(root)
            .filter((t) => !t.internal);
        const hasAnythingToExport = programsToExport.length > 0 ||
            accountsToExport.length > 0 ||
            instructionsToExport.length > 0 ||
            definedTypesToExport.length > 0;
        const ctx = {
            root,
            programsToExport,
            accountsToExport,
            instructionsToExport,
            definedTypesToExport,
            hasAnythingToExport,
        };
        const map = new RenderMap_1.RenderMap();
        if (hasAnythingToExport) {
            map.add('shared/index.ts', this.render('sharedPage.njk', ctx));
        }
        if (programsToExport.length > 0) {
            map
                .add('programs/index.ts', this.render('programsIndex.njk', ctx))
                .add('errors/index.ts', this.render('errorsIndex.njk', ctx));
        }
        if (accountsToExport.length > 0) {
            map.add('accounts/index.ts', this.render('accountsIndex.njk', ctx));
        }
        if (instructionsToExport.length > 0) {
            map.add('instructions/index.ts', this.render('instructionsIndex.njk', ctx));
        }
        if (definedTypesToExport.length > 0) {
            map.add('types/index.ts', this.render('definedTypesIndex.njk', ctx));
        }
        return map
            .add('index.ts', this.render('rootIndex.njk', ctx))
            .mergeWith(...root.programs.map((program) => (0, visitors_1.visit)(program, this)));
    }
    visitProgram(program) {
        this.program = program;
        const { name } = program;
        const pascalCaseName = (0, shared_1.pascalCase)(name);
        const renderMap = new RenderMap_1.RenderMap()
            .mergeWith(...program.accounts.map((account) => (0, visitors_1.visit)(account, this)))
            .mergeWith(...program.definedTypes.map((type) => (0, visitors_1.visit)(type, this)));
        // Internal programs are support programs that
        // were added to fill missing types or accounts.
        // They don't need to render anything else.
        if (program.internal) {
            this.program = null;
            return renderMap;
        }
        renderMap
            .mergeWith(...nodes
            .getAllInstructionsWithSubs(program, !this.options.renderParentInstructions)
            .map((ix) => (0, visitors_1.visit)(ix, this)))
            .add(`errors/${(0, shared_1.camelCase)(name)}.ts`, this.render('errorsPage.njk', {
            imports: new JavaScriptImportMap_1.JavaScriptImportMap()
                .add('umi', ['ProgramError', 'Program'])
                .toString(this.options.dependencyMap),
            program,
            errors: program.errors.map((error) => ({
                ...error,
                prefixedName: (0, shared_1.pascalCase)(program.prefix) + (0, shared_1.pascalCase)(error.name),
            })),
        }))
            .add(`programs/${(0, shared_1.camelCase)(name)}.ts`, this.render('programsPage.njk', {
            imports: new JavaScriptImportMap_1.JavaScriptImportMap()
                .add('umi', ['ClusterFilter', 'Context', 'Program', 'PublicKey'])
                .add('errors', [
                `get${pascalCaseName}ErrorFromCode`,
                `get${pascalCaseName}ErrorFromName`,
            ])
                .toString(this.options.dependencyMap),
            program,
        }));
        this.program = null;
        return renderMap;
    }
    visitAccount(account) {
        var _a;
        const isLinked = !!account.data.link;
        const typeManifest = (0, visitors_1.visit)(account, this.typeManifestVisitor);
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap().mergeWith(typeManifest.strictImports, typeManifest.serializerImports);
        if (!isLinked) {
            imports.mergeWith(typeManifest.looseImports);
        }
        imports
            .add('umi', [
            'Account',
            'assertAccountExists',
            'Context',
            'deserializeAccount',
            'Pda',
            'PublicKey',
            'publicKey',
            'RpcAccount',
            'RpcGetAccountOptions',
            'RpcGetAccountsOptions',
        ])
            .add('umiSerializers', !isLinked ? ['Serializer'] : [])
            .addAlias('umi', 'publicKey', 'toPublicKey');
        // Discriminator.
        const { discriminator } = account;
        let resolvedDiscriminator = null;
        if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'field') {
            const discriminatorField = account.data.struct.fields.find((f) => f.name === discriminator.name);
            const discriminatorValue = ((_a = discriminatorField === null || discriminatorField === void 0 ? void 0 : discriminatorField.defaultsTo) === null || _a === void 0 ? void 0 : _a.value)
                ? (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(discriminatorField.defaultsTo.value)
                : undefined;
            if (discriminatorValue) {
                imports.mergeWith(discriminatorValue.imports);
                resolvedDiscriminator = {
                    kind: 'field',
                    name: discriminator.name,
                    value: discriminatorValue.render,
                };
            }
        }
        else if ((discriminator === null || discriminator === void 0 ? void 0 : discriminator.kind) === 'size') {
            resolvedDiscriminator =
                account.size !== undefined
                    ? { kind: 'size', value: `${account.size}` }
                    : null;
        }
        // GPA Fields.
        const gpaFields = (0, shared_1.getGpaFieldsFromAccount)(account, this.byteSizeVisitor).map((gpaField) => {
            const gpaFieldManifest = (0, visitors_1.visit)(gpaField.type, this.typeManifestVisitor);
            imports.mergeWith(gpaFieldManifest.looseImports, gpaFieldManifest.serializerImports);
            return { ...gpaField, manifest: gpaFieldManifest };
        });
        let resolvedGpaFields = null;
        if (gpaFields.length > 0) {
            imports.add('umi', ['gpaBuilder']);
            resolvedGpaFields = {
                type: `{ ${gpaFields
                    .map((f) => `'${f.name}': ${f.manifest.looseType}`)
                    .join(', ')} }`,
                argument: `{ ${gpaFields
                    .map((f) => {
                    const offset = f.offset === null ? 'null' : `${f.offset}`;
                    return `'${f.name}': [${offset}, ${f.manifest.serializer}]`;
                })
                    .join(', ')} }`,
            };
        }
        // Seeds.
        const seeds = account.seeds.map((seed) => {
            if (seed.kind === 'constant') {
                const seedManifest = (0, visitors_1.visit)(seed.type, this.typeManifestVisitor);
                imports.mergeWith(seedManifest.serializerImports);
                const seedValue = seed.value;
                const valueManifest = (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(seedValue);
                seedValue.render = valueManifest.render;
                imports.mergeWith(valueManifest.imports);
                return { ...seed, typeManifest: seedManifest };
            }
            if (seed.kind === 'variable') {
                const seedManifest = (0, visitors_1.visit)(seed.type, this.typeManifestVisitor);
                imports.mergeWith(seedManifest.looseImports, seedManifest.serializerImports);
                return { ...seed, typeManifest: seedManifest };
            }
            imports
                .add('umiSerializers', 'publicKey')
                .addAlias('umiSerializers', 'publicKey', 'publicKeySerializer');
            return seed;
        });
        if (seeds.length > 0) {
            imports.add('umi', ['Pda']);
        }
        const hasVariableSeeds = account.seeds.filter((seed) => seed.kind === 'variable').length > 0;
        return new RenderMap_1.RenderMap().add(`accounts/${(0, shared_1.camelCase)(account.name)}.ts`, this.render('accountsPage.njk', {
            account,
            imports: imports.toString(this.options.dependencyMap),
            program: this.program,
            typeManifest,
            discriminator: resolvedDiscriminator,
            gpaFields: resolvedGpaFields,
            seeds,
            hasVariableSeeds,
        }));
    }
    visitInstruction(instruction) {
        var _a, _b;
        // Imports.
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap().add('umi', [
            'AccountMeta',
            'Context',
            'Signer',
            'TransactionBuilder',
            'transactionBuilder',
        ]);
        // Instruction helpers.
        const hasAccounts = instruction.accounts.length > 0;
        const hasData = !!instruction.dataArgs.link ||
            instruction.dataArgs.struct.fields.length > 0;
        const hasDataArgs = !!instruction.dataArgs.link ||
            instruction.dataArgs.struct.fields.filter((field) => { var _a; return ((_a = field.defaultsTo) === null || _a === void 0 ? void 0 : _a.strategy) !== 'omitted'; }).length > 0;
        const hasExtraArgs = !!instruction.extraArgs.link ||
            instruction.extraArgs.struct.fields.filter((field) => { var _a; return ((_a = field.defaultsTo) === null || _a === void 0 ? void 0 : _a.strategy) !== 'omitted'; }).length > 0;
        const hasAnyArgs = hasDataArgs || hasExtraArgs;
        const hasArgDefaults = Object.keys(instruction.argDefaults).length > 0;
        const hasArgResolvers = Object.values(instruction.argDefaults).some(({ kind }) => kind === 'resolver');
        const hasAccountResolvers = instruction.accounts.some(({ defaultsTo }) => (defaultsTo === null || defaultsTo === void 0 ? void 0 : defaultsTo.kind) === 'resolver');
        const hasByteResolver = ((_a = instruction.bytesCreatedOnChain) === null || _a === void 0 ? void 0 : _a.kind) === 'resolver';
        const hasRemainingAccountsResolver = ((_b = instruction.remainingAccounts) === null || _b === void 0 ? void 0 : _b.kind) === 'resolver';
        const hasResolvers = hasArgResolvers ||
            hasAccountResolvers ||
            hasByteResolver ||
            hasRemainingAccountsResolver;
        // Resolved inputs.
        const resolvedInputs = (0, visitors_1.visit)(instruction, this.resolvedInstructionInputVisitor).map((input) => {
            var _a, _b;
            if (((_a = input.defaultsTo) === null || _a === void 0 ? void 0 : _a.kind) === 'pda') {
                const { seeds } = input.defaultsTo;
                Object.keys(seeds).forEach((seed) => {
                    const seedValue = seeds[seed];
                    if (seedValue.kind !== 'value')
                        return;
                    const valueManifest = (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(seedValue.value);
                    seedValue.render = valueManifest.render;
                    imports.mergeWith(valueManifest.imports);
                });
            }
            else if (((_b = input.defaultsTo) === null || _b === void 0 ? void 0 : _b.kind) === 'value') {
                const { defaultsTo } = input;
                const valueManifest = (0, RenderJavaScriptValueNode_1.renderJavaScriptValueNode)(defaultsTo.value);
                defaultsTo.render = valueManifest.render;
                imports.mergeWith(valueManifest.imports);
            }
            return input;
        });
        const resolvedInputsWithDefaults = resolvedInputs.filter((input) => input.kind !== 'account' || input.defaultsTo !== undefined);
        if (resolvedInputsWithDefaults.length > 0) {
            imports.add('shared', 'addObjectProperty');
        }
        const accountsWithDefaults = resolvedInputsWithDefaults
            .filter((input) => input.kind === 'account')
            .map((input) => input.name);
        const argsWithDefaults = resolvedInputsWithDefaults
            .filter((input) => input.kind === 'arg')
            .map((input) => input.name);
        // Accounts.
        const accounts = instruction.accounts.map((account) => {
            const hasDefaultValue = !!account.defaultsTo;
            const resolvedAccount = resolvedInputs.find((input) => input.kind === 'account' && input.name === account.name);
            return {
                ...resolvedAccount,
                type: this.getInstructionAccountType(resolvedAccount),
                optionalSign: hasDefaultValue || account.isOptional ? '?' : '',
                hasDefaultValue,
            };
        });
        imports.mergeWith(this.getInstructionAccountImports(accounts));
        if (accounts.length > 0) {
            imports.add('shared', 'addAccountMeta');
        }
        // Data Args.
        const linkedDataArgs = !!instruction.dataArgs.link;
        const dataArgManifest = (0, visitors_1.visit)(instruction.dataArgs, this.typeManifestVisitor);
        if (linkedDataArgs || hasData) {
            imports.mergeWith(dataArgManifest.looseImports, dataArgManifest.serializerImports);
        }
        if (!linkedDataArgs) {
            imports.mergeWith(dataArgManifest.strictImports);
        }
        if (!linkedDataArgs && hasData) {
            imports.add('umiSerializers', ['Serializer']);
        }
        // Extra args.
        const linkedExtraArgs = !!instruction.extraArgs.link;
        const extraArgManifest = (0, visitors_1.visit)(instruction.extraArgs, this.typeManifestVisitor);
        imports.mergeWith(extraArgManifest.looseImports);
        // Arg defaults.
        Object.values(instruction.argDefaults).forEach((argDefault) => {
            if (argDefault.kind === 'resolver') {
                imports.add(argDefault.importFrom, (0, shared_1.camelCase)(argDefault.name));
            }
        });
        if (argsWithDefaults.length > 0) {
            imports.add('shared', ['PickPartial']);
        }
        // Bytes created on chain.
        const bytes = instruction.bytesCreatedOnChain;
        if (bytes && 'includeHeader' in bytes && bytes.includeHeader) {
            imports.add('umi', 'ACCOUNT_HEADER_SIZE');
        }
        if ((bytes === null || bytes === void 0 ? void 0 : bytes.kind) === 'account') {
            const accountName = (0, shared_1.pascalCase)(bytes.name);
            const importFrom = bytes.importFrom === 'generated'
                ? 'generatedAccounts'
                : bytes.importFrom;
            imports.add(importFrom, `get${accountName}Size`);
        }
        else if ((bytes === null || bytes === void 0 ? void 0 : bytes.kind) === 'resolver') {
            imports.add(bytes.importFrom, (0, shared_1.camelCase)(bytes.name));
        }
        // Remaining accounts.
        const { remainingAccounts } = instruction;
        if ((remainingAccounts === null || remainingAccounts === void 0 ? void 0 : remainingAccounts.kind) === 'resolver') {
            imports.add(remainingAccounts.importFrom, (0, shared_1.camelCase)(remainingAccounts.name));
        }
        // canMergeAccountsAndArgs
        let canMergeAccountsAndArgs = false;
        if (!linkedDataArgs && !linkedExtraArgs) {
            const accountsAndArgsConflicts = this.getMergeConflictsForInstructionAccountsAndArgs(instruction);
            if (accountsAndArgsConflicts.length > 0) {
                (0, logs_1.logWarn)(`[JavaScript] Accounts and args of instruction [${instruction.name}] have the following ` +
                    `conflicting attributes [${accountsAndArgsConflicts.join(', ')}]. ` +
                    `Thus, they could not be merged into a single input object. ` +
                    'You may want to rename the conflicting attributes.');
            }
            canMergeAccountsAndArgs = accountsAndArgsConflicts.length === 0;
        }
        const hasDefaultKinds = (kinds) => resolvedInputs.some((input) => input.defaultsTo && kinds.includes(input.defaultsTo.kind));
        return new RenderMap_1.RenderMap().add(`instructions/${(0, shared_1.camelCase)(instruction.name)}.ts`, this.render('instructionsPage.njk', {
            instruction,
            imports: imports.toString(this.options.dependencyMap),
            program: this.program,
            resolvedInputs,
            resolvedInputsWithDefaults,
            accountsWithDefaults,
            argsWithDefaults,
            accounts,
            needsEddsa: hasDefaultKinds(['pda']) || hasResolvers,
            needsIdentity: hasDefaultKinds(['identity']) || hasResolvers,
            needsPayer: hasDefaultKinds(['payer']) || hasResolvers,
            dataArgManifest,
            extraArgManifest,
            canMergeAccountsAndArgs,
            hasAccounts,
            hasData,
            hasDataArgs,
            hasExtraArgs,
            hasAnyArgs,
            hasArgDefaults,
            hasArgResolvers,
            hasAccountResolvers,
            hasByteResolver,
            hasRemainingAccountsResolver,
            hasResolvers,
        }));
    }
    visitDefinedType(definedType) {
        const pascalCaseName = (0, shared_1.pascalCase)(definedType.name);
        const typeManifest = (0, visitors_1.visit)(definedType, this.typeManifestVisitor);
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap()
            .mergeWithManifest(typeManifest)
            .add('umiSerializers', ['Serializer'])
            .remove('generatedTypes', [
            pascalCaseName,
            `${pascalCaseName}Args`,
            `get${pascalCaseName}Serializer`,
        ]);
        return new RenderMap_1.RenderMap().add(`types/${(0, shared_1.camelCase)(definedType.name)}.ts`, this.render('definedTypesPage.njk', {
            definedType,
            imports: imports.toString({
                ...this.options.dependencyMap,
                generatedTypes: '.',
            }),
            typeManifest,
            isDataEnum: nodes.isEnumTypeNode(definedType.data) &&
                nodes.isDataEnum(definedType.data),
        }));
    }
    get typeManifestVisitor() {
        return this.options.typeManifestVisitor;
    }
    get byteSizeVisitor() {
        return this.options.byteSizeVisitor;
    }
    get resolvedInstructionInputVisitor() {
        return this.options.resolvedInstructionInputVisitor;
    }
    getInstructionAccountType(account) {
        if (account.isPda && account.isSigner === false)
            return 'Pda';
        if (account.isSigner === 'either')
            return 'PublicKey | Pda | Signer';
        return account.isSigner ? 'Signer' : 'PublicKey | Pda';
    }
    getInstructionAccountImports(accounts) {
        const imports = new JavaScriptImportMap_1.JavaScriptImportMap();
        accounts.forEach((account) => {
            var _a, _b, _c;
            if (account.isSigner !== true && !account.isPda)
                imports.add('umi', 'PublicKey');
            if (account.isSigner !== true)
                imports.add('umi', 'Pda');
            if (account.isSigner !== false)
                imports.add('umi', 'Signer');
            if (((_a = account.defaultsTo) === null || _a === void 0 ? void 0 : _a.kind) === 'publicKey') {
                imports.add('umi', 'publicKey');
            }
            else if (((_b = account.defaultsTo) === null || _b === void 0 ? void 0 : _b.kind) === 'pda') {
                const pdaAccount = (0, shared_1.pascalCase)(account.defaultsTo.pdaAccount);
                const importFrom = account.defaultsTo.importFrom === 'generated'
                    ? 'generatedAccounts'
                    : account.defaultsTo.importFrom;
                imports.add(importFrom, `find${pdaAccount}Pda`);
                Object.values(account.defaultsTo.seeds).forEach((seed) => {
                    if (seed.kind === 'account') {
                        imports.add('umi', 'publicKey');
                    }
                });
            }
            else if (((_c = account.defaultsTo) === null || _c === void 0 ? void 0 : _c.kind) === 'resolver') {
                imports.add(account.defaultsTo.importFrom, (0, shared_1.camelCase)(account.defaultsTo.name));
            }
        });
        return imports;
    }
    getMergeConflictsForInstructionAccountsAndArgs(instruction) {
        const allNames = [
            ...instruction.accounts.map((account) => account.name),
            ...instruction.dataArgs.struct.fields.map((field) => field.name),
            ...instruction.extraArgs.struct.fields.map((field) => field.name),
        ];
        const duplicates = allNames.filter((e, i, a) => a.indexOf(e) !== i);
        return [...new Set(duplicates)];
    }
    render(template, context, options) {
        const code = (0, utils_1.resolveTemplate)(`${__dirname}/templates`, template, context, options);
        return this.options.formatCode
            ? (0, prettier_1.format)(code, this.options.prettierOptions)
            : code;
    }
}
exports.GetJavaScriptRenderMapVisitor = GetJavaScriptRenderMapVisitor;
//# sourceMappingURL=GetJavaScriptRenderMapVisitor.js.map