"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetRustRenderMapVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const visitors_1 = require("../../visitors");
const RenderMap_1 = require("../RenderMap");
const utils_1 = require("../utils");
const GetRustTypeManifestVisitor_1 = require("./GetRustTypeManifestVisitor");
const RenderRustValueNode_1 = require("./RenderRustValueNode");
const RustImportMap_1 = require("./RustImportMap");
class GetRustRenderMapVisitor extends visitors_1.BaseThrowVisitor {
    constructor(options = {}) {
        var _a, _b, _c, _d;
        super();
        this.program = null;
        this.options = {
            renderParentInstructions: (_a = options.renderParentInstructions) !== null && _a !== void 0 ? _a : false,
            dependencyMap: { ...options.dependencyMap },
            typeManifestVisitor: (_b = options.typeManifestVisitor) !== null && _b !== void 0 ? _b : new GetRustTypeManifestVisitor_1.GetRustTypeManifestVisitor(),
            byteSizeVisitor: (_c = options.byteSizeVisitor) !== null && _c !== void 0 ? _c : new visitors_1.GetByteSizeVisitor(),
            resolvedInstructionInputVisitor: (_d = options.resolvedInstructionInputVisitor) !== null && _d !== void 0 ? _d : new visitors_1.GetResolvedInstructionInputsVisitor(),
        };
    }
    visitRoot(root) {
        var _a, _b;
        (_b = (_a = this.byteSizeVisitor).registerDefinedTypes) === null || _b === void 0 ? void 0 : _b.call(_a, nodes.getAllDefinedTypes(root));
        const programsToExport = root.programs.filter((p) => !p.internal);
        const accountsToExport = nodes
            .getAllAccounts(root)
            .filter((a) => !a.internal);
        const instructionsToExport = nodes
            .getAllInstructionsWithSubs(root, !this.options.renderParentInstructions)
            .filter((i) => !i.internal);
        const definedTypesToExport = nodes
            .getAllDefinedTypes(root)
            .filter((t) => !t.internal);
        const hasAnythingToExport = programsToExport.length > 0 ||
            accountsToExport.length > 0 ||
            instructionsToExport.length > 0 ||
            definedTypesToExport.length > 0;
        const ctx = {
            root,
            programsToExport,
            accountsToExport,
            instructionsToExport,
            definedTypesToExport,
            hasAnythingToExport,
        };
        const map = new RenderMap_1.RenderMap();
        if (programsToExport.length > 0) {
            map
                .add('programs.rs', this.render('programsMod.njk', ctx))
                .add('errors/mod.rs', this.render('errorsMod.njk', ctx));
        }
        if (accountsToExport.length > 0) {
            map.add('accounts/mod.rs', this.render('accountsMod.njk', ctx));
        }
        if (instructionsToExport.length > 0) {
            map.add('instructions/mod.rs', this.render('instructionsMod.njk', ctx));
        }
        if (definedTypesToExport.length > 0) {
            map
                .add('types/mod.rs', this.render('definedTypesMod.njk', ctx))
                .add('types/helper/mod.rs', this.render('sharedPage.njk', ctx));
        }
        return map
            .add('mod.rs', this.render('rootMod.njk', ctx))
            .mergeWith(...root.programs.map((program) => (0, visitors_1.visit)(program, this)));
    }
    visitProgram(program) {
        this.program = program;
        const { name } = program;
        const renderMap = new RenderMap_1.RenderMap()
            .mergeWith(...program.accounts.map((account) => (0, visitors_1.visit)(account, this)))
            .mergeWith(...program.definedTypes.map((type) => (0, visitors_1.visit)(type, this)));
        // Internal programs are support programs that
        // were added to fill missing types or accounts.
        // They don't need to render anything else.
        if (program.internal) {
            this.program = null;
            return renderMap;
        }
        renderMap
            .mergeWith(...nodes
            .getAllInstructionsWithSubs(program, !this.options.renderParentInstructions)
            .map((ix) => (0, visitors_1.visit)(ix, this)))
            .add(`errors/${(0, shared_1.snakeCase)(name)}.rs`, this.render('errorsPage.njk', {
            imports: new RustImportMap_1.RustImportMap().toString(this.options.dependencyMap),
            program,
            errors: program.errors.map((error) => ({
                ...error,
                prefixedName: (0, shared_1.pascalCase)(program.prefix) + (0, shared_1.pascalCase)(error.name),
            })),
        }));
        this.program = null;
        return renderMap;
    }
    visitAccount(account) {
        const typeManifest = (0, visitors_1.visit)(account, this.typeManifestVisitor);
        // Seeds.
        const seedsImports = new RustImportMap_1.RustImportMap();
        const seeds = account.seeds.map((seed) => {
            if (seed.kind === 'constant') {
                const seedManifest = (0, visitors_1.visit)(seed.type, this.typeManifestVisitor);
                const seedValue = seed.value;
                const valueManifest = (0, RenderRustValueNode_1.renderRustValueNode)(seedValue, true);
                seedValue.render = valueManifest.render;
                seedsImports.mergeWith(valueManifest.imports);
                return { ...seed, typeManifest: seedManifest };
            }
            if (seed.kind === 'variable') {
                const seedManifest = (0, visitors_1.visit)(seed.type, this.typeManifestVisitor);
                seedsImports.mergeWith(seedManifest.imports);
                return { ...seed, typeManifest: seedManifest };
            }
            return seed;
        });
        const hasVariableSeeds = account.seeds.filter((seed) => seed.kind === 'variable').length > 0;
        const { imports } = typeManifest;
        if (hasVariableSeeds) {
            imports.mergeWith(seedsImports);
        }
        return new RenderMap_1.RenderMap().add(`accounts/${(0, shared_1.snakeCase)(account.name)}.rs`, this.render('accountsPage.njk', {
            account,
            imports: imports
                .remove(`generatedAccounts::${(0, shared_1.pascalCase)(account.name)}`)
                .toString(this.options.dependencyMap),
            program: this.program,
            typeManifest,
            seeds,
            hasVariableSeeds,
        }));
    }
    visitInstruction(instruction) {
        // Imports.
        const imports = new RustImportMap_1.RustImportMap();
        imports.add(['borsh::BorshDeserialize', 'borsh::BorshSerialize']);
        // canMergeAccountsAndArgs
        const accountsAndArgsConflicts = this.getConflictsForInstructionAccountsAndArgs(instruction);
        if (accountsAndArgsConflicts.length > 0) {
            (0, shared_1.logWarn)(`[Rust] Accounts and args of instruction [${instruction.name}] have the following ` +
                `conflicting attributes [${accountsAndArgsConflicts.join(', ')}]. ` +
                `Thus, the conflicting arguments will be suffixed with "_arg". ` +
                'You may want to rename the conflicting attributes.');
        }
        // Instruction args.
        const instructionArgs = [];
        let hasArgs = false;
        let hasOptional = false;
        instruction.dataArgs.struct.fields.forEach((field) => {
            var _a, _b, _c;
            this.typeManifestVisitor.parentName =
                (0, shared_1.pascalCase)(instruction.dataArgs.name) + (0, shared_1.pascalCase)(field.name);
            this.typeManifestVisitor.nestedStruct = true;
            const manifest = (0, visitors_1.visit)(field.child, this.typeManifestVisitor);
            imports.mergeWith(manifest.imports);
            const innerOptionType = nodes.isOptionTypeNode(field.child)
                ? manifest.type.slice('Option<'.length, -1)
                : null;
            this.typeManifestVisitor.parentName = null;
            this.typeManifestVisitor.nestedStruct = false;
            let renderValue = null;
            if (field.defaultsTo) {
                const { imports: argImports, render: value } = (0, RenderRustValueNode_1.renderRustValueNode)(field.defaultsTo.value);
                imports.mergeWith(argImports);
                renderValue = value;
            }
            else {
                hasArgs = true;
            }
            hasOptional = hasOptional || ((_a = field.defaultsTo) === null || _a === void 0 ? void 0 : _a.strategy) === 'optional';
            const name = accountsAndArgsConflicts.includes(field.name)
                ? `${field.name}_arg`
                : field.name;
            instructionArgs.push({
                name,
                type: manifest.type,
                default: ((_b = field.defaultsTo) === null || _b === void 0 ? void 0 : _b.strategy) === 'omitted',
                optional: ((_c = field.defaultsTo) === null || _c === void 0 ? void 0 : _c.strategy) === 'optional',
                innerOptionType,
                value: renderValue,
            });
        });
        const typeManifest = (0, visitors_1.visit)(instruction, this.typeManifestVisitor);
        return new RenderMap_1.RenderMap().add(`instructions/${(0, shared_1.snakeCase)(instruction.name)}.rs`, this.render('instructionsPage.njk', {
            instruction,
            imports: imports
                .remove(`generatedInstructions::${(0, shared_1.pascalCase)(instruction.name)}`)
                .toString(this.options.dependencyMap),
            instructionArgs,
            hasArgs,
            hasOptional,
            program: this.program,
            typeManifest,
        }));
    }
    visitDefinedType(definedType) {
        const typeManifest = (0, visitors_1.visit)(definedType, this.typeManifestVisitor);
        const imports = new RustImportMap_1.RustImportMap().mergeWithManifest(typeManifest);
        return new RenderMap_1.RenderMap().add(`types/${(0, shared_1.snakeCase)(definedType.name)}.rs`, this.render('definedTypesPage.njk', {
            definedType,
            imports: imports
                .remove(`generatedTypes::${(0, shared_1.pascalCase)(definedType.name)}`)
                .toString(this.options.dependencyMap),
            typeManifest,
        }));
    }
    get typeManifestVisitor() {
        return this.options.typeManifestVisitor;
    }
    get byteSizeVisitor() {
        return this.options.byteSizeVisitor;
    }
    get resolvedInstructionInputVisitor() {
        return this.options.resolvedInstructionInputVisitor;
    }
    getConflictsForInstructionAccountsAndArgs(instruction) {
        const allNames = [
            ...instruction.accounts.map((account) => account.name),
            ...instruction.dataArgs.struct.fields.map((field) => field.name),
        ];
        const duplicates = allNames.filter((e, i, a) => a.indexOf(e) !== i);
        return [...new Set(duplicates)];
    }
    render(template, context, options) {
        const code = (0, utils_1.resolveTemplate)(`${__dirname}/templates`, template, context, options);
        return code;
    }
}
exports.GetRustRenderMapVisitor = GetRustRenderMapVisitor;
//# sourceMappingURL=GetRustRenderMapVisitor.js.map