"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetRustTypeManifestVisitor = void 0;
/* eslint-disable @typescript-eslint/no-unused-vars */
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const visitors_1 = require("../../visitors");
const RustImportMap_1 = require("./RustImportMap");
class GetRustTypeManifestVisitor {
    constructor() {
        this.parentName = null;
        this.nestedStruct = false;
        this.inlineStruct = false;
    }
    visitRoot() {
        throw new Error('Cannot get type manifest for root node. Please select a child node.');
    }
    visitProgram() {
        throw new Error('Cannot get type manifest for program node. Please select a child node.');
    }
    visitAccount(account) {
        this.parentName = (0, shared_1.pascalCase)(account.name);
        const manifest = (0, visitors_1.visit)(account.data, this);
        manifest.imports.add(['borsh::BorshSerialize', 'borsh::BorshDeserialize']);
        this.parentName = null;
        return {
            ...manifest,
            type: `#[derive(BorshSerialize, BorshDeserialize, Clone, Debug, Eq, PartialEq)]\n${manifest.type}`,
        };
    }
    visitAccountData(accountData) {
        return (0, visitors_1.visit)(accountData.struct, this);
    }
    visitInstruction(instruction) {
        return (0, visitors_1.visit)(instruction.dataArgs, this);
    }
    visitInstructionAccount() {
        throw new Error('Cannot get type manifest for instruction account node. Please select a another node.');
    }
    visitInstructionDataArgs(instructionDataArgs) {
        this.parentName = (0, shared_1.pascalCase)(instructionDataArgs.name);
        const manifest = (0, visitors_1.visit)(instructionDataArgs.struct, this);
        this.parentName = null;
        return manifest;
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        this.parentName = (0, shared_1.pascalCase)(instructionExtraArgs.name);
        const manifest = (0, visitors_1.visit)(instructionExtraArgs.struct, this);
        this.parentName = null;
        return manifest;
    }
    visitDefinedType(definedType) {
        this.parentName = (0, shared_1.pascalCase)(definedType.name);
        const manifest = (0, visitors_1.visit)(definedType.data, this);
        this.parentName = null;
        manifest.imports.add(['borsh::BorshSerialize', 'borsh::BorshDeserialize']);
        const traits = [
            'BorshSerialize',
            'BorshDeserialize',
            'Clone',
            'Debug',
            'Eq',
            'PartialEq',
        ];
        const isScalarEnum = nodes.isEnumTypeNode(definedType.data) &&
            nodes.isScalarEnum(definedType.data);
        if (isScalarEnum) {
            traits.push('PartialOrd', 'Hash');
        }
        return {
            ...manifest,
            type: `#[derive(${traits.join(', ')})]\n${manifest.type}`,
            nestedStructs: manifest.nestedStructs.map((struct) => `#[derive(${traits.join(', ')})]\n${struct}`),
        };
    }
    visitError() {
        throw new Error('Cannot get type manifest for error node.');
    }
    visitArrayType(arrayType) {
        const childManifest = (0, visitors_1.visit)(arrayType.child, this);
        if (arrayType.size.kind === 'fixed') {
            return {
                ...childManifest,
                type: `[${childManifest.type}; ${arrayType.size.value}]`,
            };
        }
        if (arrayType.size.kind === 'prefixed' &&
            arrayType.size.prefix.format === 'u32' &&
            arrayType.size.prefix.endian === 'le') {
            return {
                ...childManifest,
                type: `Vec<${childManifest.type}>`,
            };
        }
        if (arrayType.size.kind === 'remainder') {
            childManifest.imports.add('crate::types::helper::RemainderVec');
            return {
                ...childManifest,
                type: `RemainderVec<${childManifest.type}>`,
            };
        }
        // TODO: Add to the Rust validator.
        throw new Error('Array size not supported by Borsh');
    }
    visitLinkType(linkType) {
        const pascalCaseDefinedType = (0, shared_1.pascalCase)(linkType.name);
        const importFrom = linkType.importFrom === 'generated'
            ? 'generatedTypes'
            : linkType.importFrom;
        return {
            imports: new RustImportMap_1.RustImportMap().add(`${importFrom}::${pascalCaseDefinedType}`),
            type: pascalCaseDefinedType,
            nestedStructs: [],
        };
    }
    visitEnumType(enumType) {
        const { parentName } = this;
        if (!parentName) {
            // TODO: Add to the Rust validator.
            throw new Error('Enum type must have a parent name.');
        }
        const variants = enumType.variants.map((variant) => (0, visitors_1.visit)(variant, this));
        const variantNames = variants.map((variant) => variant.type).join('\n');
        const mergedManifest = this.mergeManifests(variants);
        return {
            ...mergedManifest,
            type: `pub enum ${(0, shared_1.pascalCase)(parentName)} {\n${variantNames}\n}`,
        };
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        const name = (0, shared_1.pascalCase)(enumEmptyVariantType.name);
        return {
            type: `${name},`,
            imports: new RustImportMap_1.RustImportMap(),
            nestedStructs: [],
        };
    }
    visitEnumStructVariantType(enumStructVariantType) {
        const name = (0, shared_1.pascalCase)(enumStructVariantType.name);
        const originalParentName = this.parentName;
        if (!originalParentName) {
            throw new Error('Enum struct variant type must have a parent name.');
        }
        this.inlineStruct = true;
        this.parentName = (0, shared_1.pascalCase)(originalParentName) + name;
        const typeManifest = (0, visitors_1.visit)(enumStructVariantType.struct, this);
        this.inlineStruct = false;
        this.parentName = originalParentName;
        return {
            ...typeManifest,
            type: `${name} ${typeManifest.type},`,
        };
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        const name = (0, shared_1.pascalCase)(enumTupleVariantType.name);
        const originalParentName = this.parentName;
        if (!originalParentName) {
            throw new Error('Enum struct variant type must have a parent name.');
        }
        this.parentName = (0, shared_1.pascalCase)(originalParentName) + name;
        const childManifest = (0, visitors_1.visit)(enumTupleVariantType.tuple, this);
        this.parentName = originalParentName;
        return {
            ...childManifest,
            type: `${name}${childManifest.type},`,
        };
    }
    visitMapType(mapType) {
        const key = (0, visitors_1.visit)(mapType.key, this);
        const value = (0, visitors_1.visit)(mapType.value, this);
        const mergedManifest = this.mergeManifests([key, value]);
        mergedManifest.imports.add('std::collections::HashMap');
        return {
            ...mergedManifest,
            type: `HashMap<${key.type}, ${value.type}>`,
        };
    }
    visitOptionType(optionType) {
        const childManifest = (0, visitors_1.visit)(optionType.child, this);
        if (optionType.prefix.format === 'u8' &&
            optionType.prefix.endian === 'le') {
            return {
                ...childManifest,
                type: `Option<${childManifest.type}>`,
            };
        }
        // TODO: Add to the Rust validator.
        throw new Error('Option size not supported by Borsh');
    }
    visitSetType(setType) {
        const childManifest = (0, visitors_1.visit)(setType.child, this);
        childManifest.imports.add('std::collections::HashSet');
        return {
            ...childManifest,
            type: `HashSet<${childManifest.type}>`,
        };
    }
    visitStructType(structType) {
        const { parentName } = this;
        if (!parentName) {
            // TODO: Add to the Rust validator.
            throw new Error('Struct type must have a parent name.');
        }
        const fields = structType.fields.map((field) => (0, visitors_1.visit)(field, this));
        const fieldTypes = fields.map((field) => field.type).join('\n');
        const mergedManifest = this.mergeManifests(fields);
        if (this.nestedStruct) {
            return {
                ...mergedManifest,
                type: (0, shared_1.pascalCase)(parentName),
                nestedStructs: [
                    ...mergedManifest.nestedStructs,
                    `pub struct ${(0, shared_1.pascalCase)(parentName)} {\n${fieldTypes}\n}`,
                ],
            };
        }
        if (this.inlineStruct) {
            return { ...mergedManifest, type: `{\n${fieldTypes}\n}` };
        }
        return {
            ...mergedManifest,
            type: `pub struct ${(0, shared_1.pascalCase)(parentName)} {\n${fieldTypes}\n}`,
        };
    }
    visitStructFieldType(structFieldType) {
        const originalParentName = this.parentName;
        const originalInlineStruct = this.inlineStruct;
        const originalNestedStruct = this.nestedStruct;
        if (!originalParentName) {
            throw new Error('Struct field type must have a parent name.');
        }
        this.parentName =
            (0, shared_1.pascalCase)(originalParentName) + (0, shared_1.pascalCase)(structFieldType.name);
        this.nestedStruct = true;
        this.inlineStruct = false;
        const fieldManifest = (0, visitors_1.visit)(structFieldType.child, this);
        this.parentName = originalParentName;
        this.inlineStruct = originalInlineStruct;
        this.nestedStruct = originalNestedStruct;
        const fieldName = (0, shared_1.snakeCase)(structFieldType.name);
        const docblock = this.createDocblock(structFieldType.docs);
        return {
            ...fieldManifest,
            type: this.inlineStruct
                ? `${docblock}${fieldName}: ${fieldManifest.type},`
                : `${docblock}pub ${fieldName}: ${fieldManifest.type},`,
        };
    }
    visitTupleType(tupleType) {
        const children = tupleType.children.map((item) => (0, visitors_1.visit)(item, this));
        const mergedManifest = this.mergeManifests(children);
        return {
            ...mergedManifest,
            type: `(${children.map((item) => item.type).join(', ')})`,
        };
    }
    visitBoolType(boolType) {
        if (boolType.size.format === 'u8' && boolType.size.endian === 'le') {
            return { type: 'bool', imports: new RustImportMap_1.RustImportMap(), nestedStructs: [] };
        }
        // TODO: Add to the Rust validator.
        throw new Error('Bool size not supported by Borsh');
    }
    visitBytesType(bytesType) {
        const arrayType = nodes.arrayTypeNode(nodes.numberTypeNode('u8'), {
            size: bytesType.size,
        });
        return (0, visitors_1.visit)(arrayType, this);
    }
    visitNumberType(numberType) {
        if (numberType.endian === 'le') {
            return {
                type: numberType.format,
                imports: new RustImportMap_1.RustImportMap(),
                nestedStructs: [],
            };
        }
        // TODO: Add to the Rust validator.
        throw new Error('Number endianness not supported by Borsh');
    }
    visitNumberWrapperType(numberWrapperType) {
        return (0, visitors_1.visit)(numberWrapperType.number, this);
    }
    visitPublicKeyType() {
        return {
            type: 'Pubkey',
            imports: new RustImportMap_1.RustImportMap().add('solana_program::pubkey::Pubkey'),
            nestedStructs: [],
        };
    }
    visitStringType(stringType) {
        if (stringType.size.kind === 'prefixed' &&
            stringType.size.prefix.format === 'u32' &&
            stringType.size.prefix.endian === 'le') {
            return {
                type: 'String',
                imports: new RustImportMap_1.RustImportMap(),
                nestedStructs: [],
            };
        }
        if (stringType.size.kind === 'fixed') {
            return {
                type: `[u8; ${stringType.size.value}]`,
                imports: new RustImportMap_1.RustImportMap(),
                nestedStructs: [],
            };
        }
        if (stringType.size.kind === 'remainder') {
            return {
                type: `&str`,
                imports: new RustImportMap_1.RustImportMap(),
                nestedStructs: [],
            };
        }
        // TODO: Add to the Rust validator.
        throw new Error('String size not supported by Borsh');
    }
    mergeManifests(manifests) {
        return {
            imports: new RustImportMap_1.RustImportMap().mergeWith(...manifests.map((td) => td.imports)),
            nestedStructs: manifests.flatMap((m) => m.nestedStructs),
        };
    }
    createDocblock(docs) {
        if (docs.length <= 0)
            return '';
        const lines = docs.map((doc) => `/// ${doc}`);
        return `${lines.join('\n')}\n`;
    }
}
exports.GetRustTypeManifestVisitor = GetRustTypeManifestVisitor;
//# sourceMappingURL=GetRustTypeManifestVisitor.js.map