"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RenderRustVisitor = void 0;
const child_process_1 = require("child_process");
const logs_1 = require("../../shared/logs");
const visitors_1 = require("../../visitors");
const WriteRenderMapVisitor_1 = require("../WriteRenderMapVisitor");
const utils_1 = require("../utils");
const GetRustRenderMapVisitor_1 = require("./GetRustRenderMapVisitor");
class RenderRustVisitor extends visitors_1.BaseThrowVisitor {
    constructor(path, options = {}) {
        super();
        this.path = path;
        this.options = options;
    }
    visitRoot(root) {
        // Validate nodes.
        // visit(
        //   root,
        //   new ThrowValidatorItemsVisitor(
        //     new GetRustValidatorBagVisitor(),
        //     this.options.throwLevel
        //   )
        // );
        var _a;
        // Delete existing generated folder.
        if ((_a = this.options.deleteFolderBeforeRendering) !== null && _a !== void 0 ? _a : true) {
            (0, utils_1.deleteFolder)(this.path);
        }
        // Render the new files.
        (0, visitors_1.visit)(root, new WriteRenderMapVisitor_1.WriteRenderMapVisitor(new GetRustRenderMapVisitor_1.GetRustRenderMapVisitor(this.options), this.path));
        // format the code
        if (this.options.formatCode) {
            if (this.options.crateFolder) {
                runFormatter('cargo-fmt', [
                    '--manifest-path',
                    `${this.options.crateFolder}/Cargo.toml`,
                ]);
            }
            else {
                (0, logs_1.logWarn)('No crate folder specified, skipping formatting.');
            }
        }
    }
}
exports.RenderRustVisitor = RenderRustVisitor;
function runFormatter(cmd, args) {
    var _a;
    const { stdout, stderr, error } = (0, child_process_1.spawnSync)(cmd, args);
    if ((_a = error === null || error === void 0 ? void 0 : error.message) === null || _a === void 0 ? void 0 : _a.includes('ENOENT')) {
        (0, logs_1.logWarn)(`Could not find ${cmd}, skipping formatting.`);
        return;
    }
    if (stdout.length > 0) {
        (0, logs_1.logWarn)(`(cargo-fmt) ${stdout || error}`);
    }
    if (stderr.length > 0) {
        (0, logs_1.logError)(`(cargo-fmt) ${stderr || error}`);
    }
}
//# sourceMappingURL=RenderRustVisitor.js.map