"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shouldLogLevel = exports.getLevelIndex = exports.logDebug = exports.logTrace = exports.logInfo = exports.logWarn = exports.logError = exports.LOG_LEVELS = void 0;
/* eslint-disable no-console */
const chalk_1 = __importDefault(require("chalk"));
const DEFAULT_LOG_LEVEL = 'info';
exports.LOG_LEVELS = ['debug', 'trace', 'info', 'warn', 'error'];
const logError = (message, hint) => {
    if (!(0, exports.shouldLogLevel)('error'))
        return;
    logMessageAndHint(`${chalk_1.default.bgRed.black(' Error ')} ${chalk_1.default.redBright(message)}`, hint);
};
exports.logError = logError;
const logWarn = (message, hint) => {
    if (!(0, exports.shouldLogLevel)('warn'))
        return;
    logMessageAndHint(`${chalk_1.default.bgYellow.black(' Warning ')} ${chalk_1.default.yellow(message)}`, hint);
};
exports.logWarn = logWarn;
const logInfo = (message, hint) => {
    if (!(0, exports.shouldLogLevel)('info'))
        return;
    logMessageAndHint(`${chalk_1.default.bgBlue.black(' Info ')} ${chalk_1.default.blue(message)}`, hint);
};
exports.logInfo = logInfo;
const logTrace = (message, hint) => {
    if (!(0, exports.shouldLogLevel)('trace'))
        return;
    logMessageAndHint(`${chalk_1.default.bgWhite.black(' Trace ')} ${chalk_1.default.white(message)}`, hint);
};
exports.logTrace = logTrace;
const logDebug = (message, debug, hint) => {
    if (!(0, exports.shouldLogLevel)('debug'))
        return;
    logMessageAndHint(`${chalk_1.default.bgMagentaBright.black(' Debug ')} ${chalk_1.default.magentaBright(message)}`, hint, false);
    if (debug !== undefined)
        console.log(debug);
    console.log('\n');
};
exports.logDebug = logDebug;
const getLevelIndex = (level) => exports.LOG_LEVELS.indexOf(level);
exports.getLevelIndex = getLevelIndex;
const shouldLogLevel = (level) => (0, exports.getLevelIndex)(level) >= (0, exports.getLevelIndex)(getMinLogLevel());
exports.shouldLogLevel = shouldLogLevel;
const getMinLogLevel = () => {
    const level = process.env.LOG;
    if (level && exports.LOG_LEVELS.includes(level)) {
        return level;
    }
    return DEFAULT_LOG_LEVEL;
};
const logMessageAndHint = (message, hint, lineJump = true) => {
    console.log(message + getDimmedHint(hint) + (lineJump ? '\n' : ''));
};
const getDimmedHint = (hint, prefix = '|> ') => {
    if (!hint)
        return '';
    const hintArray = (hint !== null && hint !== void 0 ? hint : '').split('\n').filter((line) => !!line);
    return chalk_1.default.dim(hintArray.map((h) => `\n${prefix}${h}`).join(''));
};
//# sourceMappingURL=logs.js.map