"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseNodeVisitor = void 0;
const Visitor_1 = require("./Visitor");
const nodes = __importStar(require("../nodes"));
class BaseNodeVisitor {
    visitRoot(root) {
        return nodes.rootNode(root.programs
            .map((program) => (0, Visitor_1.visit)(program, this))
            .filter(nodes.assertNodeFilter(nodes.assertProgramNode)));
    }
    visitProgram(program) {
        return nodes.programNode({
            ...program,
            accounts: program.accounts
                .map((account) => (0, Visitor_1.visit)(account, this))
                .filter(nodes.assertNodeFilter(nodes.assertAccountNode)),
            instructions: program.instructions
                .map((instruction) => (0, Visitor_1.visit)(instruction, this))
                .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)),
            definedTypes: program.definedTypes
                .map((type) => (0, Visitor_1.visit)(type, this))
                .filter(nodes.assertNodeFilter(nodes.assertDefinedTypeNode)),
            errors: program.errors
                .map((error) => (0, Visitor_1.visit)(error, this))
                .filter(nodes.assertNodeFilter(nodes.assertErrorNode)),
        });
    }
    visitAccount(account) {
        const data = (0, Visitor_1.visit)(account.data, this);
        nodes.assertAccountDataNode(data);
        const seeds = account.seeds
            .map((seed) => {
            if (seed.kind !== 'variable')
                return seed;
            const newType = (0, Visitor_1.visit)(seed.type, this);
            nodes.assertTypeNode(newType);
            return { ...seed, type: newType };
        })
            .filter((s) => s !== null);
        return nodes.accountNode({ ...account, data, seeds });
    }
    visitAccountData(accountData) {
        const struct = (0, Visitor_1.visit)(accountData.struct, this);
        nodes.assertStructTypeNode(struct);
        const link = accountData.link ? (0, Visitor_1.visit)(accountData.link, this) : undefined;
        if (link !== undefined)
            nodes.assertLinkTypeNode(link);
        return nodes.accountDataNode({ ...accountData, struct, link });
    }
    visitInstruction(instruction) {
        const dataArgs = (0, Visitor_1.visit)(instruction.dataArgs, this);
        nodes.assertInstructionDataArgsNode(dataArgs);
        const extraArgs = (0, Visitor_1.visit)(instruction.extraArgs, this);
        nodes.assertInstructionExtraArgsNode(extraArgs);
        return nodes.instructionNode({
            ...instruction,
            dataArgs,
            extraArgs,
            accounts: instruction.accounts
                .map((account) => (0, Visitor_1.visit)(account, this))
                .filter(nodes.assertNodeFilter(nodes.assertInstructionAccountNode)),
            subInstructions: instruction.subInstructions
                .map((ix) => (0, Visitor_1.visit)(ix, this))
                .filter(nodes.assertNodeFilter(nodes.assertInstructionNode)),
        });
    }
    visitInstructionAccount(instructionAccount) {
        return instructionAccount;
    }
    visitInstructionDataArgs(instructionDataArgs) {
        const struct = (0, Visitor_1.visit)(instructionDataArgs.struct, this);
        nodes.assertStructTypeNode(struct);
        const link = instructionDataArgs.link
            ? (0, Visitor_1.visit)(instructionDataArgs.link, this)
            : undefined;
        if (link !== undefined)
            nodes.assertLinkTypeNode(link);
        return nodes.instructionDataArgsNode({
            ...instructionDataArgs,
            struct,
            link,
        });
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        const struct = (0, Visitor_1.visit)(instructionExtraArgs.struct, this);
        nodes.assertStructTypeNode(struct);
        const link = instructionExtraArgs.link
            ? (0, Visitor_1.visit)(instructionExtraArgs.link, this)
            : undefined;
        if (link !== undefined)
            nodes.assertLinkTypeNode(link);
        return nodes.instructionExtraArgsNode({
            ...instructionExtraArgs,
            struct,
            link,
        });
    }
    visitDefinedType(definedType) {
        const data = (0, Visitor_1.visit)(definedType.data, this);
        nodes.assertTypeNode(data);
        return nodes.definedTypeNode({ ...definedType, data });
    }
    visitError(error) {
        return error;
    }
    visitArrayType(arrayType) {
        const child = (0, Visitor_1.visit)(arrayType.child, this);
        nodes.assertTypeNode(child);
        return nodes.arrayTypeNode(child, { ...arrayType });
    }
    visitLinkType(linkType) {
        return linkType;
    }
    visitEnumType(enumType) {
        return nodes.enumTypeNode(enumType.variants
            .map((variant) => (0, Visitor_1.visit)(variant, this))
            .filter(nodes.assertNodeFilter(nodes.assertEnumVariantTypeNode)), { ...enumType });
    }
    visitEnumEmptyVariantType(enumEmptyVariantType) {
        return enumEmptyVariantType;
    }
    visitEnumStructVariantType(enumStructVariantType) {
        const newStruct = (0, Visitor_1.visit)(enumStructVariantType.struct, this);
        nodes.assertStructTypeNode(newStruct);
        return nodes.enumStructVariantTypeNode(enumStructVariantType.name, newStruct);
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        const newTuple = (0, Visitor_1.visit)(enumTupleVariantType.tuple, this);
        nodes.assertTupleTypeNode(newTuple);
        return nodes.enumTupleVariantTypeNode(enumTupleVariantType.name, newTuple);
    }
    visitMapType(mapType) {
        const key = (0, Visitor_1.visit)(mapType.key, this);
        const value = (0, Visitor_1.visit)(mapType.value, this);
        nodes.assertTypeNode(key);
        nodes.assertTypeNode(value);
        return nodes.mapTypeNode(key, value, { ...mapType });
    }
    visitOptionType(optionType) {
        const child = (0, Visitor_1.visit)(optionType.child, this);
        nodes.assertTypeNode(child);
        return nodes.optionTypeNode(child, { ...optionType });
    }
    visitSetType(setType) {
        const child = (0, Visitor_1.visit)(setType.child, this);
        nodes.assertTypeNode(child);
        return nodes.setTypeNode(child, { ...setType });
    }
    visitStructType(structType) {
        return nodes.structTypeNode(structType.fields
            .map((field) => (0, Visitor_1.visit)(field, this))
            .filter(nodes.assertNodeFilter(nodes.assertStructFieldTypeNode)));
    }
    visitStructFieldType(structFieldType) {
        const child = (0, Visitor_1.visit)(structFieldType.child, this);
        nodes.assertTypeNode(child);
        return nodes.structFieldTypeNode({ ...structFieldType, child });
    }
    visitTupleType(tupleType) {
        return nodes.tupleTypeNode(tupleType.children
            .map((child) => (0, Visitor_1.visit)(child, this))
            .filter(nodes.assertNodeFilter(nodes.assertTypeNode)));
    }
    visitBoolType(boolType) {
        return boolType;
    }
    visitBytesType(bytesType) {
        return bytesType;
    }
    visitNumberType(numberType) {
        return numberType;
    }
    visitNumberWrapperType(numberWrapperType) {
        const number = (0, Visitor_1.visit)(numberWrapperType.number, this);
        nodes.assertNumberTypeNode(number);
        return nodes.numberWrapperTypeNode(number, numberWrapperType.wrapper);
    }
    visitPublicKeyType(publicKeyType) {
        return publicKeyType;
    }
    visitStringType(stringType) {
        return stringType;
    }
}
exports.BaseNodeVisitor = BaseNodeVisitor;
//# sourceMappingURL=BaseNodeVisitor.js.map