"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseVoidVisitor = void 0;
const Visitor_1 = require("./Visitor");
class BaseVoidVisitor {
    visitRoot(root) {
        root.programs.forEach((program) => (0, Visitor_1.visit)(program, this));
    }
    visitProgram(program) {
        program.accounts.forEach((account) => (0, Visitor_1.visit)(account, this));
        program.instructions.forEach((instruction) => (0, Visitor_1.visit)(instruction, this));
        program.definedTypes.forEach((type) => (0, Visitor_1.visit)(type, this));
        program.errors.forEach((type) => (0, Visitor_1.visit)(type, this));
    }
    visitAccount(account) {
        (0, Visitor_1.visit)(account.data, this);
        account.seeds.forEach((seed) => {
            if (seed.kind !== 'variable')
                return;
            (0, Visitor_1.visit)(seed.type, this);
        });
    }
    visitAccountData(accountData) {
        (0, Visitor_1.visit)(accountData.struct, this);
        if (accountData.link)
            (0, Visitor_1.visit)(accountData.link, this);
    }
    visitInstruction(instruction) {
        (0, Visitor_1.visit)(instruction.dataArgs, this);
        (0, Visitor_1.visit)(instruction.extraArgs, this);
        instruction.accounts.forEach((account) => (0, Visitor_1.visit)(account, this));
        instruction.subInstructions.forEach((ix) => (0, Visitor_1.visit)(ix, this));
    }
    visitInstructionAccount(
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    instructionAccount) {
        //
    }
    visitInstructionDataArgs(instructionDataArgs) {
        (0, Visitor_1.visit)(instructionDataArgs.struct, this);
        if (instructionDataArgs.link)
            (0, Visitor_1.visit)(instructionDataArgs.link, this);
    }
    visitInstructionExtraArgs(instructionExtraArgs) {
        (0, Visitor_1.visit)(instructionExtraArgs.struct, this);
        if (instructionExtraArgs.link)
            (0, Visitor_1.visit)(instructionExtraArgs.link, this);
    }
    visitDefinedType(definedType) {
        (0, Visitor_1.visit)(definedType.data, this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitError(error) {
        //
    }
    visitArrayType(arrayType) {
        (0, Visitor_1.visit)(arrayType.child, this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitLinkType(linkType) {
        //
    }
    visitEnumType(enumType) {
        enumType.variants.forEach((variant) => (0, Visitor_1.visit)(variant, this));
    }
    visitEnumEmptyVariantType(
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    enumEmptyVariantType) {
        //
    }
    visitEnumStructVariantType(enumStructVariantType) {
        (0, Visitor_1.visit)(enumStructVariantType.struct, this);
    }
    visitEnumTupleVariantType(enumTupleVariantType) {
        (0, Visitor_1.visit)(enumTupleVariantType.tuple, this);
    }
    visitMapType(mapType) {
        (0, Visitor_1.visit)(mapType.key, this);
        (0, Visitor_1.visit)(mapType.value, this);
    }
    visitOptionType(optionType) {
        (0, Visitor_1.visit)(optionType.child, this);
    }
    visitSetType(setType) {
        (0, Visitor_1.visit)(setType.child, this);
    }
    visitStructType(structType) {
        structType.fields.forEach((field) => (0, Visitor_1.visit)(field, this));
    }
    visitStructFieldType(structFieldType) {
        (0, Visitor_1.visit)(structFieldType.child, this);
    }
    visitTupleType(tupleType) {
        tupleType.children.forEach((child) => (0, Visitor_1.visit)(child, this));
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitBoolType(boolType) {
        //
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitBytesType(bytesType) {
        //
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitNumberType(numberType) {
        //
    }
    visitNumberWrapperType(numberWrapperType) {
        (0, Visitor_1.visit)(numberWrapperType.number, this);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitPublicKeyType(publicKeyType) {
        //
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    visitStringType(stringType) {
        //
    }
}
exports.BaseVoidVisitor = BaseVoidVisitor;
//# sourceMappingURL=BaseVoidVisitor.js.map