"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.visit = void 0;
const shared_1 = require("../shared");
function visit(node, visitor) {
    switch (node.kind) {
        case 'rootNode':
            return visitor.visitRoot(node);
        case 'programNode':
            return visitor.visitProgram(node);
        case 'accountNode':
            return visitor.visitAccount(node);
        case 'accountDataNode':
            return visitor.visitAccountData(node);
        case 'instructionNode':
            return visitor.visitInstruction(node);
        case 'instructionAccountNode':
            return visitor.visitInstructionAccount(node);
        case 'instructionDataArgsNode':
            return visitor.visitInstructionDataArgs(node);
        case 'instructionExtraArgsNode':
            return visitor.visitInstructionExtraArgs(node);
        case 'definedTypeNode':
            return visitor.visitDefinedType(node);
        case 'errorNode':
            return visitor.visitError(node);
        case 'arrayTypeNode':
            return visitor.visitArrayType(node);
        case 'linkTypeNode':
            return visitor.visitLinkType(node);
        case 'enumTypeNode':
            return visitor.visitEnumType(node);
        case 'enumEmptyVariantTypeNode':
            return visitor.visitEnumEmptyVariantType(node);
        case 'enumStructVariantTypeNode':
            return visitor.visitEnumStructVariantType(node);
        case 'enumTupleVariantTypeNode':
            return visitor.visitEnumTupleVariantType(node);
        case 'mapTypeNode':
            return visitor.visitMapType(node);
        case 'optionTypeNode':
            return visitor.visitOptionType(node);
        case 'setTypeNode':
            return visitor.visitSetType(node);
        case 'structTypeNode':
            return visitor.visitStructType(node);
        case 'structFieldTypeNode':
            return visitor.visitStructFieldType(node);
        case 'tupleTypeNode':
            return visitor.visitTupleType(node);
        case 'boolTypeNode':
            return visitor.visitBoolType(node);
        case 'bytesTypeNode':
            return visitor.visitBytesType(node);
        case 'numberTypeNode':
            return visitor.visitNumberType(node);
        case 'numberWrapperTypeNode':
            return visitor.visitNumberWrapperType(node);
        case 'publicKeyTypeNode':
            return visitor.visitPublicKeyType(node);
        case 'stringTypeNode':
            return visitor.visitStringType(node);
        default:
            const nodeAsNever = node;
            throw new shared_1.KinobiError(`Unrecognized node [${nodeAsNever.kind}]`);
    }
}
exports.visit = visit;
//# sourceMappingURL=Visitor.js.map