import type { ConfigureOptions } from 'nunjucks';
import * as nodes from '../../nodes';
import { ImportFrom } from '../../shared';
import { BaseThrowVisitor, ResolvedInstructionInput, Visitor } from '../../visitors';
import { RenderMap } from '../RenderMap';
import { RustTypeManifest } from './GetRustTypeManifestVisitor';
export type GetRustRenderMapOptions = {
    renderParentInstructions?: boolean;
    dependencyMap?: Record<ImportFrom, string>;
    typeManifestVisitor?: Visitor<RustTypeManifest> & {
        parentName: string | null;
        nestedStruct: boolean;
    };
    byteSizeVisitor?: Visitor<number | null> & {
        registerDefinedTypes?: (definedTypes: nodes.DefinedTypeNode[]) => void;
    };
    resolvedInstructionInputVisitor?: Visitor<ResolvedInstructionInput[]>;
};
export declare class GetRustRenderMapVisitor extends BaseThrowVisitor<RenderMap> {
    readonly options: Required<GetRustRenderMapOptions>;
    private program;
    constructor(options?: GetRustRenderMapOptions);
    visitRoot(root: nodes.RootNode): RenderMap;
    visitProgram(program: nodes.ProgramNode): RenderMap;
    visitAccount(account: nodes.AccountNode): RenderMap;
    visitInstruction(instruction: nodes.InstructionNode): RenderMap;
    visitDefinedType(definedType: nodes.DefinedTypeNode): RenderMap;
    get typeManifestVisitor(): Visitor<RustTypeManifest> & {
        parentName: string | null;
        nestedStruct: boolean;
    };
    get byteSizeVisitor(): Visitor<number | null> & {
        registerDefinedTypes?: ((definedTypes: nodes.DefinedTypeNode[]) => void) | undefined;
    };
    get resolvedInstructionInputVisitor(): Visitor<ResolvedInstructionInput[]>;
    protected getConflictsForInstructionAccountsAndArgs(instruction: nodes.InstructionNode): string[];
    protected render(template: string, context?: object, options?: ConfigureOptions): string;
}
