import * as nodes from '../../nodes';
import { Visitor } from '../../visitors';
import { RustImportMap } from './RustImportMap';
export type RustTypeManifest = {
    type: string;
    imports: RustImportMap;
    nestedStructs: string[];
};
export declare class GetRustTypeManifestVisitor implements Visitor<RustTypeManifest> {
    parentName: string | null;
    nestedStruct: boolean;
    private inlineStruct;
    visitRoot(): RustTypeManifest;
    visitProgram(): RustTypeManifest;
    visitAccount(account: nodes.AccountNode): RustTypeManifest;
    visitAccountData(accountData: nodes.AccountDataNode): RustTypeManifest;
    visitInstruction(instruction: nodes.InstructionNode): RustTypeManifest;
    visitInstructionAccount(): RustTypeManifest;
    visitInstructionDataArgs(instructionDataArgs: nodes.InstructionDataArgsNode): RustTypeManifest;
    visitInstructionExtraArgs(instructionExtraArgs: nodes.InstructionExtraArgsNode): RustTypeManifest;
    visitDefinedType(definedType: nodes.DefinedTypeNode): RustTypeManifest;
    visitError(): RustTypeManifest;
    visitArrayType(arrayType: nodes.ArrayTypeNode): RustTypeManifest;
    visitLinkType(linkType: nodes.LinkTypeNode): RustTypeManifest;
    visitEnumType(enumType: nodes.EnumTypeNode): RustTypeManifest;
    visitEnumEmptyVariantType(enumEmptyVariantType: nodes.EnumEmptyVariantTypeNode): RustTypeManifest;
    visitEnumStructVariantType(enumStructVariantType: nodes.EnumStructVariantTypeNode): RustTypeManifest;
    visitEnumTupleVariantType(enumTupleVariantType: nodes.EnumTupleVariantTypeNode): RustTypeManifest;
    visitMapType(mapType: nodes.MapTypeNode): RustTypeManifest;
    visitOptionType(optionType: nodes.OptionTypeNode): RustTypeManifest;
    visitSetType(setType: nodes.SetTypeNode): RustTypeManifest;
    visitStructType(structType: nodes.StructTypeNode): RustTypeManifest;
    visitStructFieldType(structFieldType: nodes.StructFieldTypeNode): RustTypeManifest;
    visitTupleType(tupleType: nodes.TupleTypeNode): RustTypeManifest;
    visitBoolType(boolType: nodes.BoolTypeNode): RustTypeManifest;
    visitBytesType(bytesType: nodes.BytesTypeNode): RustTypeManifest;
    visitNumberType(numberType: nodes.NumberTypeNode): RustTypeManifest;
    visitNumberWrapperType(numberWrapperType: nodes.NumberWrapperTypeNode): RustTypeManifest;
    visitPublicKeyType(): RustTypeManifest;
    visitStringType(stringType: nodes.StringTypeNode): RustTypeManifest;
    protected mergeManifests(manifests: RustTypeManifest[]): Pick<RustTypeManifest, 'imports' | 'nestedStructs'>;
    protected createDocblock(docs: string[]): string;
}
